package widebase.rpc.table

import org.joda.time. { LocalDate, YearMonth, Years }

import widebase.db.table. { PartitionMap, Table }
import widebase.rpc.interface.ClientInterface

/** Loads tables from remote database.
 *
 * @param proxy to client interface
 * @param authId of client
 *
 * @author myst3r10n
 */
class RemoteTableLoad(proxy: ClientInterface, authId: String) {

  import vario.filter.StreamFilter.StreamFilter

  /** Load table from remote database.
    *
    * @param name of table
    * @param filter self-explanatory
    * @param segmented segment key
    *
    * @return [[widebase.db.Table]]
   */
  def apply(name: String, filter: StreamFilter = props.filters.deserialize)
    (implicit segmented: String = null): Table =
    Table.fromBytes(proxy.load(authId, name)(segmented), filter)

  /** Load directory table from remote database.
    *
    * @param name of table
    * @param filter self-explanatory
    * @param parted partition name
    * @param segmented segment key
    *
    * @return [[widebase.db.column.Table]]
   */
  def dir(name: String, filter: StreamFilter = props.filters.deserialize)
    (implicit parted: String = null, segmented: String = null) = {

    val partition: String =
      if(parted == null)
        ""
      else
        parted

    Table.fromBytes(proxy.loadDir(authId, name)(partition, segmented), filter)

  }

  /** Load directory table from remote database by [[org.joda.time.LocalDate]].
    *
    * @param name of table
    * @param filter self-explanatory
    * @param from the [[org.joda.time.LocalDate]] from
    * @param till the [[org.joda.time.LocalDate]] till
    * @param segmented segment key
    *
    * @return [[widebase.db.column.Table]]
   */
  def dates(
    name: String,
    from: LocalDate,
    till: LocalDate,
    filter: StreamFilter = props.filters.deserialize)
    (implicit segmented: String = null) = {

    var parts = new PartitionMap

    var i = from

    while(i.compareTo(till) < 0) {

      val partition = i.toString

      if(proxy.findDir(authId, name)(partition, segmented))
        parts +=
          partition ->
          Table.fromBytes(
            proxy.loadDir(authId, name)(partition, segmented),
            filter)

      i = i.plusDays(1)

    }

    parts

  }

  /** Load directory table from remote database by [[scala.Int]].
    *
    * @param name of table
    * @param filter self-explanatory
    * @param from the [[scala.Int]] from
    * @param till the [[scala.Int]] till
    * @param segmented segment key
    *
    * @return [[widebase.db.column.Table]]
   */
  def ints(
    name: String,
    from: Int,
    till: Int,
    filter: StreamFilter = props.filters.deserialize)
    (implicit segmented: String = null) = {

    var parts = new PartitionMap

    var i = from

    for(i <- from to till) {

      val partition = i.toString

      if(proxy.findDir(authId, name)(partition, segmented))
        parts +=
          partition ->
          Table.fromBytes(
            proxy.loadDir(authId, name)(partition, segmented),
            filter)


    }

    parts

  }

  /** Load directory table from remote database by [[org.joda.time.YearMonth]].
    *
    * @param name of table
    * @param filter self-explanatory
    * @param from the [[org.joda.time.YearMonth]] from
    * @param till the [[org.joda.time.YearMonth]] till
    * @param segmented segment key
    *
    * @return [[widebase.db.column.Table]]
   */
  def months(
    name: String,
    from: YearMonth,
    till: YearMonth,
    filter: StreamFilter = props.filters.deserialize)
    (implicit segmented: String = null) = {

    var parts = new PartitionMap

    var i = from

    while(i.compareTo(till) < 0) {

      val partition = i.toString

      if(proxy.findDir(authId, name)(partition, segmented))
        parts +=
          partition ->
          Table.fromBytes(
            proxy.loadDir(authId, name)(partition, segmented),
            filter)

      i = i.plusMonths(1)

    }

    parts

  }

  /** Load directory table from remote database by [[org.joda.time.Years]].
    *
    * @param name of table
    * @param filter self-explanatory
    * @param from the [[org.joda.time.Years]] from
    * @param till the [[org.joda.time.Years]] till
    * @param segmented segment key
    *
    * @return [[widebase.db.column.Table]]
   */
  def years(
    name: String,
    from: Years,
    till: Years,
    filter: StreamFilter = props.filters.deserialize)
    (implicit segmented: String = null) = {

    var parts = new PartitionMap

    var i = from

    while(i.compareTo(till) < 0) {

      val partition = i.toString

      if(proxy.findDir(authId, name)(partition, segmented))
        parts +=
          partition ->
          Table.fromBytes(
            proxy.loadDir(authId, name)(partition, segmented),
            filter)

      i = i.plus(1)

    }

    parts

  }
}

